{% extends 'players/base.html' %}

{% block content %}
    <h1>Welcome to Kurono!</h1>
    <p>Kurono is an online multiplayer game, where players create programs to play on their behalf.</p>
    <p>Are you interested in learning more about Kurono and would like to try it out? <a href="https://www.codeforlife.education/play/kurono/">Find out more and register your interest on our main website.</a></p>
{% endblock %}
