{% extends 'players/base.html' %}

{% block content %}
<div class="row">
    <div class="col-md-6 col-md-offset-3">
        <h1>Add a new game</h1>
        <form action="" method="post">
            {% csrf_token %}
            <div class="form-actions">
                {{ form }}
                <input type="submit" value="Submit" class="btn btn-primary">
            </div>
        </form>
    </div>
</div>

{% endblock %}
