# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2019-07-03 11:51
from __future__ import unicode_literals

from django.db import migrations


def reset_tokens(apps, schema_editor):
    Game = apps.get_model("aimmo", "Game")
    for game in Game.objects.all():
        game.auth_token = ""
        game.save()


def reverse(apps, schema_editor):
    pass


class Migration(migrations.Migration):

    dependencies = [("aimmo", "0010_alter_game_token")]

    operations = [migrations.RunPython(reset_tokens, reverse_code=reverse)]
