DEFAULT_LEVEL_SETTINGS = {
    "TARGET_NUM_CELLS_PER_AVATAR": 0,
    "TARGET_NUM_SCORE_LOCATIONS_PER_AVATAR": 0,
    "SCORE_DESPAWN_CHANCE": 0,
    "TARGET_NUM_PICKUPS_PER_AVATAR": 0,
    "PICKUP_SPAWN_CHANCE": 0,
    "NO_FOG_OF_WAR_DISTANCE": 1000,
    "PARTIAL_FOG_OF_WAR_DISTANCE": 1000,
}
